@@ -1,6 +1,6 @@
 //! Various assertions.
 
-/// Runs a block, returning a ProgramResult.
+/// Runs a block, returning a [anchor_lang::prelude::ProgramResult].
 #[macro_export]
 macro_rules! test_assertion {
     ($body: block) => {
@@ -72,7 +72,7 @@ macro_rules! format_err {
 /// ```
 #[macro_export]
 macro_rules! program_err {
-    ($error:tt $(,)?) => {
+    ($error:ident $(,)?) => {
         Err(crate::ErrorCode::$error.into())
     };
 }
@@ -85,13 +85,64 @@ macro_rules! log_code_location {
     };
 }
 
+/// Unwraps a block which returns an [Option].
+///
+/// This is useful for running checked math expressions within a function which returns [Result].
+///
+/// # Examples
+///
+/// ```
+/// # use anchor_lang::prelude::*;
+/// # #[macro_use] extern crate vipers; fn main() -> ProgramResult {
+/// let result = unwrap_opt_block!({
+///     let one: u64 = 1;
+///     let three: u64 = 3;
+///     let four = one.checked_add(three)?;
+///     four.checked_add(3)
+/// });
+/// assert_eq!(result, 7);
+/// # Ok(())
+/// # }
+/// ```
+#[macro_export]
+macro_rules! unwrap_opt_block {
+    ($body:block $($arg:tt)*) => {
+        $crate::unwrap_opt!(
+            #[allow(clippy::redundant_closure_call)]
+            (|| { $body } )() $($arg)*)
+    };
+}
+
+/// Unwraps the result of a block of checked integer math.
+///
+/// # Example
+///
+/// ```
+/// # use anchor_lang::prelude::*;
+/// # #[macro_use] extern crate vipers; fn main() -> ProgramResult {
+/// let result = unwrap_checked!({
+///   let one: u64 = 1;
+///   let three: u64 = 3;
+///   one.checked_add(three)
+/// });
+/// assert_eq!(result, 4);
+/// # Ok(())
+/// # }
+/// ```
+#[macro_export]
+macro_rules! unwrap_checked {
+    ($body:block $(,)?) => {
+        $crate::unwrap_opt_block!($body, $crate::VipersError::IntegerOverflow)
+    };
+}
+
 /// Throws an error.
 ///
 /// # Example
 ///
 /// ```
 /// # use anchor_lang::prelude::*;
-/// # impl From<ErrorCode> for ProgramError { fn from(code: ErrorCode) -> Self { ProgramError::Custom(10) } }
+/// # #[error]
 /// # pub enum ErrorCode { MyError }
 /// # #[macro_use] extern crate vipers; fn main() -> ProgramResult {
 /// let fail = false;
@@ -327,8 +378,7 @@ macro_rules! assert_is_zero_token_account {
     };
     ($token_account: expr, $msg: literal $(,)?) => {
         $crate::assert_is_zero_token_account!(
-            $account_a,
-            $account_b,
+            $token_account,
             $crate::VipersError::TokenAccountIsNonZero,
             &*format!("Token account is non-zero: {}", $msg),
         );
@@ -415,7 +465,7 @@ macro_rules! assert_keys_neq {
 /// ```
 #[macro_export]
 macro_rules! unwrap_or_err {
-    ($option:expr, $error:tt $(,)?) => {
+    ($option:expr, $error:ident $(,)?) => {
         $option.ok_or_else(|| -> ProgramError { crate::ErrorCode::$error.into() })?
     };
 }
@@ -458,7 +508,7 @@ macro_rules! unwrap_int {
 /// ```
 #[macro_export]
 macro_rules! try_or_err {
-    ($result:expr, $error:tt $(,)?) => {
+    ($result:expr, $error:ident $(,)?) => {
         $result.map_err(|_| -> ProgramError { crate::ErrorCode::$error.into() })?
     };
 }
@@ -566,6 +616,13 @@ macro_rules! invariant {
 /// ```
 #[macro_export]
 macro_rules! unwrap_opt {
+    ($option: expr $(,)?) => {
+        $crate::unwrap_opt!(
+            $option,
+            $crate::VipersError::OptionUnwrapFailed,
+            $crate::format_err!($crate::VipersError::OptionUnwrapFailed)
+        )
+    };
     ($option: expr, $err_code: ident $(,)?) => {
         $crate::unwrap_opt!($option, crate::ErrorCode::$err_code)
     };
@@ -578,8 +635,9 @@ macro_rules! unwrap_opt {
     ($option:expr, $err:expr, $msg: expr $(,)?) => {
         $option.ok_or_else(|| -> ProgramError {
             msg!("Option unwrap failed: {:?}", $err);
+            msg!(stringify!($option));
             $crate::log_code_location!();
-            $crate::VipersError::OptionUnwrapFailed.into()
+            $err.into()
         })?
     };
 }
@@ -600,10 +658,16 @@ macro_rules! assert_keys {
 
 #[cfg(test)]
 mod tests {
+    use crate::prelude::*;
     use anchor_lang::prelude::*;
     use anchor_spl::token;
     use spl_associated_token_account::get_associated_token_address;
 
+    #[error]
+    pub enum ErrorCode {
+        MyError,
+    }
+
     #[account]
     #[derive(Default)]
     struct TestData {
@@ -658,4 +722,73 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn test_unwrap_checked() -> ProgramResult {
+        assert_throws!(
+            {
+                unwrap_checked!({
+                    let one: u64 = 1;
+                    let four = one.checked_add(u64::MAX)?;
+                    four.checked_add(3)
+                });
+            },
+            VipersError::IntegerOverflow
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_unwrap_opt_block() {
+        assert_throws!(
+            {
+                unwrap_opt_block!(
+                    {
+                        let one: u64 = 1;
+                        one.checked_add(u64::MAX)
+                    },
+                    ErrorCode::MyError
+                );
+            },
+            ErrorCode::MyError
+        );
+    }
+
+    #[test]
+    #[allow(clippy::eq_op)]
+    fn test_invariant() {
+        assert_does_not_throw!({
+            invariant!(1 == 1, ErrorCode::MyError);
+        });
+        assert_throws!(
+            {
+                invariant!(1 == 2);
+            },
+            VipersError::InvariantFailed
+        );
+        assert_throws!(
+            {
+                invariant!(1 == 2, "this is stupid");
+            },
+            VipersError::InvariantFailed
+        );
+        assert_throws!(
+            {
+                invariant!(1 == 2, ErrorCode::MyError);
+            },
+            ErrorCode::MyError
+        );
+        assert_throws!(
+            {
+                invariant!(1 == 2, ErrorCode::MyError);
+            },
+            ErrorCode::MyError
+        );
+        assert_throws!(
+            {
+                invariant!(1 == 2, ErrorCode::MyError, "this is wack");
+            },
+            ErrorCode::MyError
+        );
+    }
 }
